*-----------------------------------------------------------------------------
* RmcFwhT1.prg (adapted by R. Seiler from RmcDemo.prg)
*
*              This is a limited test version, just to test a few
*              of problematic calls to Rmchart dll functions.
*
*              April 14, 2008
*
*              For use with the RMChart graphics application developed by
*              Rainer Morgen and available for download at: www.rmchart.com
*
* See comments at bottom of this prg regarding prior errors and their causes.
*
* Testing done here with an adaptation of Morgen's demo function mnSingleBars().
*-----------------------------------------------------------------------------
/*
Procedures and functions in this prg:

  // The main entry into the app...
PROCEDURE Main()

  // Create the app's menu system...
STATIC FUNCTION BuildMenu()

  // Test for creating a bar graph...
STATIC FUNCTION mnSingleBars()

  // Create child window...
STATIC FUNCTION MakeChWnd()

  // Command exiting of the app...
PROCEDURE cmdClose()

  // Exit the app...
EXIT PROCEDURE EndAll()

FUNCTION IsError(nError)

*/ //------------------- End of Functions List ------------------------------//

#include "FiveWin.ch"
#include "Rmchart.ch"
#include "DllxHb.ch"
#include "cstruct.ch"
#include "wintypes.ch"

#define True   1
#define False  0

#define TAB   CHR(9)
#command QUIT => ( PostQuitMessage( 0 ), __Quit() ) // suggested by Enrico

#define DC_CALL_STD          0x0020   // Needed if using xHb's DLLCALL()
                                      // instead of FWH's DLL32.

STATIC ;
       cPath     := ""         ,;
       hRmcDll   := 0          ,;
       lDirect   := .f.        ,;
       nCtrlId   := 0          ,;
       nWleft    := 0          ,;
       nWtop     := 0          ,;
       hWnd      := 0          ,;
       hChWnd    := 0          ,;
       oBrush                  ,;
       oWnd                    ,;
       oWndCh                  

// The following lines setup a C structure for oData used in mnSingleBars()...

pragma pack(4) 

typedef struct {;
  CTYPE_DOUBLE nDouble[5];
} DataStructD5

typedef struct {;
  CTYPE_LONG nLong[5];
} ColorStructL5

*------------------------------------------
PROCEDURE Main()

   cPath  := CurDrive()+":\"+CurDir()
   nWleft := 6
   nWtop  := 3

   hRmcDll  := LoadLibrary( "RMCHART.DLL" )


   SET _3DLOOK ON

   DEFINE BRUSH oBrush COLOR GetSysColor(15)

   DEFINE WINDOW oWnd FROM 3, 6 TO 45, 120 ;
      TITLE "RmChart Test" ;
      BRUSH oBrush         ;
      MENU  BuildMenu()    ;
      MDI

   SET MESSAGE OF oWnd TO "RMChart message bar" ;
      CENTERED DATE KEYBOARD 2007
   
   ACTIVATE WINDOW oWnd    ;
     ON INIT( MakeChWnd(), hWnd := oWnd:hWnd, nCtrlID := oWnd:hWnd ) 


RETURN
*------------------------------
  // Create the app's main menu system...
STATIC FUNCTION BuildMenu()
   local oMenu,lToggle := .f.

   local sRMCfile := "SimplBar.rmc"

   static lHide := .f.


   MENU oMenu 2007
      MENUITEM "&Test Demo"  ;
        MESSAGE "Test demo."
      MENU
         MENUITEM   "&Bars - diff. colors and tooltips"        ;
            ACTION  ( KillChart(), mnBarsImage() )             ;
            MESSAGE "Bars with diff. colors and tooltips"

         MENUITEM   "&Exit"                    ;
            ACTION  ( cmdClose() )             ;
            MESSAGE "Exit program."
      ENDMENU
   ENDMENU

RETURN oMenu
//---------------------------------------------------------------------------//
  // This test function for creating a bar graph necessarily uses a
  // building block approach, calling a series of Rmchart functions in sequence
  // because that is the way Rmchart creates a graph, one step at a time...
STATIC FUNCTION mnBarsImage()
    LOCAL nRetVal := 0
    LOCAL sLabels := ""
    LOCAL oData  IS DataStructD5
    LOCAL oColor IS ColorStructL5
    
    nCtrlID  := hWnd  // Better to use some predefined CONSTANT ID.
    
    //************** Create the chart **********************
   nRetVal = RMC_CreateChart( ;
                 hWnd                , ;  // nParentHandle
                 nCtrlID             , ;  // nCtrlID
                 0                   , ;  // nTop
                 0                   , ;  // nLeft
                 600                 , ;  // nWidth
                 500                 , ;  // nHeight
                 ColorTransparent    , ;  // nBackColor
                 RMC_CTRLSTYLEIMAGE  , ;  // nCtrlStyle
                 False               , ;  // nExportOnly
                 ""                  , ;  // sBgImage  // "seasky.jpg" - background jpeg, not needed for this test
                 ""                  , ;  // sFontName
                 100                 , ;  // nToolTipwidth
                 0                     ;  // nBitmapBKColor
                 )

    If nRetVal < 0 ; IsError(41) ; RETURN nil ; EndIf

    //************** Add Region 1 *****************************
   nRetVal = RMC_AddRegion( ;
                 nCtrlID             , ;  // nCtrlID
                 5                   , ;  // nLeft
                 5                   , ;  // nTop
                 -5                  , ;  // nWidth
                 -5                  , ;  // nHeight
                 ""                  , ;  // sFooter
                 False                 ;  // nShowBorder
                 )

    If nRetVal < 0 ; IsError(42) ; RETURN nil ; EndIf

    //************** Add grid to region 1 *****************************
   nRetVal = RMC_AddGrid( ;
                 nCtrlID             , ;  // nCtrlID
                 1                   , ;  // nRegion
                 ColorTransparent    , ;  // nGridBackColor
                 False               , ;  // nAsGradient
                 0                   , ;  // nLeft
                 0                   , ;  // nTop
                 0                   , ;  // nWidth
                 0                   , ;  // nHeight
                                       ;  // nBiColor
                 )


    If nRetVal < 0 ; IsError(43) ; RETURN nil ; EndIf

    //************** Add data axis to region 1 *****************************
   nRetVal = RMC_AddDataAxis( ;
                 nCtrlID             , ;  // nCtrlID
                 1                   , ;  // nRegion
                 RMC_DATAAXISBOTTOM  , ;  // nAlignment
                 0.0                 , ;  // nMinValue -DOUBLE so MUST have decimal!!
                 100.0               , ;  // nMaxValue -DOUBLE so MUST have decimal!!
                 11                  , ;  // nTickCount
                 8                   , ;  // nFontsize
                 ColorGreen          , ;  // nTextColor
                 ColorGreen          , ;  // nLineColor
                 RMC_LINESTYLEDOT    , ;  // nLinestyle
                 0                   , ;  // nDecimalDigits
                 ""                  , ;  // sUnit
                 ""                  , ;  // sText
                                     , ;  // sLabels
                                       ;  // nLabelAlignment
                 )


    If nRetVal < 0 ; IsError(44) ; RETURN nil ; EndIf
    //************** Add label axis to region 1 *****************************

    sLabels = "Label 1*Label 2*Label 3*Label 4*Label 5"

   nRetVal = RMC_AddLabelAxis( ;
                 nCtrlID             , ;  // nCtrlID
                 1                   , ;  // nRegion
                 sLabels             , ;  // sLabels (separated by asterisk)
                 1                   , ;  // nCount
                 5                   , ;  // ntickCount
                 RMC_LABELAXISBOTTOM , ;  // nAlignment
                 8                   , ;  // nFontSize
                 ColorBlack          , ;  // ntextColor
                 RMC_TEXTCENTER      , ;  // nTextAlignment
                 ColorGreen          , ;  // nLineColor
                 RMC_LINESTYLENONE   , ;  // nLineStyle
                 ""                    ;  // sText
                 )


    If nRetVal < 0 ; IsError(45) ; RETURN nil ; EndIf
    //************** Add Series 1 to region 1 *******************************
    //****** Read data values ******

    oData:nDouble[1] := 50.0    // Your data goes into these 5 slots
    oData:nDouble[2] := 70.0
    oData:nDouble[3] := 40.0
    oData:nDouble[4] := 60.0
    oData:nDouble[5] := 30.0

   nRetVal = RMC_AddBarSeries( ;
                 nCtrlID             , ;  // nCtrlID
                 1                   , ;  // nRegion
                 oData[1]            , ;  // nFirstDataValue
                 5                   , ;  // nDataValuesCount
                 RMC_BARSINGLE       , ;  // nType
                 RMC_BAR_FLAT_GRADIENT2 , ;  // nStyle
                 True                , ;  // nIsLucent
                 ColorTransparent    , ;  // nColor
                 False               , ;  // nIsHorizontal
                 1                   , ;  // nWhichAxis
                 RMC_VLABEL_NONE     , ;  // nValuelabelOn
                 1                   , ;  // nPointsPerColumn
                 RMC_HATCHBRUSH_OFF    ;  // nHatchMode
                 )


    If nRetVal < 0 ; IsError(46) ; RETURN nil ; EndIf
    
    // Lets set some CustomToolTipTexts: (these don't work, don't know why)
    RMC_SetCustomToolTipText( nCtrlID, 1, 1, 1, ;
                              "First Bar" + Chr(13) + Chr(10) + ;
                              "Value: 50" + Chr(13) + Chr(10) + ;
                              "Color: Blue" )
    RMC_SetCustomToolTipText( nCtrlID, 1, 1, 2, ;
                              "Second Bar" + Chr(13) + Chr(10) + ;
                              "Value: 70" + Chr(13) + Chr(10) + ;
                              "Color: Green" )
    RMC_SetCustomToolTipText( nCtrlID, 1, 1, 3, ;
                              "Third Bar" + Chr(13) + Chr(10) + ;
                              "Value: 40" + Chr(13) + Chr(10) + ;
                              "Color: Red" )
    RMC_SetCustomToolTipText( nCtrlID, 1, 1, 4, ;
                              "Fourth Bar" + Chr(13) + Chr(10) + ;
                              "Value: 60" + Chr(13) + Chr(10) + ;
                              "Color: Yellow" )
    // Last bar gets no CustomText, so default ToolTipText (=Value) is used.
    
    // Switch ToolTips on
    RMC_AddToolTips( nCtrlID, hWnd, 100 )
*    RMC_SetToolTipWidth( nCtrlID, 100 )    // Alternate way to switch on TT.
 
    nRetVal = RMC_Draw( nCtrlID )
    If nRetVal < 0 ; IsError(47) ; RETURN nil ; EndIf

RETURN nil


*-------------------------------------
STATIC FUNCTION IsError(nError)
IF EMPTY(nError)
   nError := 0
ENDIF

   MsgAlert("An error occurred at: "+LTRIM(STR(nError)) +CRLF+;
             "Please make a note of this number.")

   // Each error number can be traced to a specific place in this prg.

RETURN nil
*--------------------------------------
  // Create child window upon which chart can optionally be placed...
STATIC FUNCTION MakeChWnd()

   DEFINE WINDOW oWndCh MDICHILD OF oWnd    ;
          FROM 2, 2 TO 37, 110 ;
          STYLE nOr(WS_CHILD)  ;
          COLOR "N/W" 

   ACTIVATE WINDOW oWndCh ;
     ON INIT (hChWnd := oWndCh:hWnd)

RETURN nil
*sb----------------------------
  // Mandate app exit...
PROCEDURE cmdClose()

    IF !EMPTY(oWndCh)
       oWndCh:End()
    ENDIF
    IF !EMPTY(oWnd)
       oBrush:End()
       oWnd:End()    // This automatically forces execution of the EXIT PROC
    ENDIF

RETURN
*--------------------------------------
EXIT PROCEDURE EndAll()

    IF !EMPTY(oWndCh)
       oWndCh:End()
    ENDIF
    IF !EMPTY(oWnd)
       oBrush:End()
       oWnd:End()
    ENDIF

*    MsgInfo("Executing exit procedure.")

     // Always delete the chart when closing the app!!
    DLLCALL(hRmcDll,DC_CALL_STD, ;
     "RMC_DELETECHART", nCtrlId )

    FreeLibrary( hRmcDll )
    ResAllFree()
    Close Databases
    Clear Memory

RETURN
*----------------------------
STATIC FUNCTION KillChart()

  IF !EMPTY(nCtrlId)
     DLLCALL(hRmcDll,DC_CALL_STD, ;
        "RMC_DELETECHART", nCtrlId )
     nCtrlId := 0
  ENDIF

RETURN nil
*----------------------------

// Active versions utilizing DLLxHb.ch without data types...

// For indirect use of xHarbour's DLLCALL() via DLLxHb.ch...
// (These are in Rmc4xHb.prg, but we're not using that for this test.)

// Note: dllcall.c automatically analyzes and determines the data types,
//       so data types are not specified. However, one problem with this is
//       that the data for a DOUBLE _must_ be passed with a decimal point
//       or it will be erroneously classified as LONG causing a GPF. So must
//       be careful with doubles to pass 0 as 0.0 and pass 100 as 100.0 .

// To see what the actual data types are for each of the functions below, one
// can refer to RMChart.inc (used with the PowerBasic version of RMChart).

DLLxHb FUNCTION RMC_CREATECHART( ;           // RMC_CreateChart
                 nParentHndl, ;     // HWND
                 nCtrlId, ;
                 nX, ;
                 nY, ;
                 nWidth, ;
                 nHeight, ;
                 nBackColor, ;
                 nCtrlStyle, ;
                 nExportOnly, ;
                 sBgImage, ;
                 sFontName, ;
                 nToolTipWidth, ;
                 nBitmapBKColor ;
                 ) LIB hRmcDll

DLLxHb FUNCTION RMC_ADDREGION( ;             // RMC_AddRegion
                 nCtrlID, ;
                 nLeft, ;
                 nTop , ;
                 nWidth, ;
                 nHeight, ;
                 sFooter, ;
                 nShowBorder ;
                 ) LIB hRmcDll 

DLLxHb FUNCTION RMC_ADDBARSERIES(  ;         // RMC_AddBarSeries 
                 nCtrlId             , ;
                 nRegion             , ;
                 @nFirstDataValue    , ;
                 nDataValuesCount    , ;
                 nType               , ;
                 nStyle              , ;
                 nIsLucent           , ;
                 nColor              , ;
                 nIsHorizontal       , ;
                 nWhichDataAxis      , ;
                 nValueLabelOn       , ;
                 nPointsPerColumn    , ;
                 nHatchMode ) LIB hRmcDll 

DLLxHb FUNCTION RMC_ADDDATAAXIS( ;           // RMC_AddDataAxis
                 nCtrlID, ;
                 nRegion, ;
                 nAlignment, ;
                 nMinValue, ;
                 nMaxValue, ;
                 nTickCount, ;
                 nFontsize, ;
                 nTextColor, ;
                 nLineColor, ;
                 nLineStyle, ;
                 nDecimalDigits, ;
                 sUnit, ;
                 sText, ;
                 sLabels, ;
                 nLabelAlignment ;
                 ) LIB hRmcDll

DLLxHb FUNCTION RMC_ADDGRID( ;               // RMC_AddGrid
                 nCtrlID, ;
                 nRegion, ;
                 nBackColor, ;
                 nAsGradient, ;
                 nLeft, ;
                 nTop, ;
                 nWidth, ;
                 nHeight, ;
                 nBiColor ;
                 ) LIB hRmcDll 

DLLxHb FUNCTION RMC_ADDLABELAXIS( ;          // RMC_AddLabelAxis
                 nCtrlID, ;
                 nRegion, ;
                 sLabel, ;     
                 nCount, ;
                 nTickCount, ;
                 nAlignment, ;
                 nFontsize, ;
                 nTextColor, ;
                 nTextAlignment, ;
                 nLineColor, ;
                 nLineStyle, ;
                 sText ;                 
                 ) LIB hRmcDll 

DLLxHb FUNCTION RMC_ADDTOOLTIPS( ;   // RMC_AddToolTips
                 nCtrlID, ;
                 hWnd, ;
                 nToolTipWidth ;
                 ) LIB hRmcDll 

DLLxHb FUNCTION RMC_DRAW( ;                  // RMC_Draw
                 nCtrlID ;
                 ) LIB hRmcDll 

DLLxHb FUNCTION RMC_SETCUSTOMTOOLTIPTEXT( ;  // RMC_SetCustomToolTipText
                  nCtrlId, ;
                  nRegion, ;
                  nSeries, ;
                  nDataIndex, ;
                  sText ;
                  ) LIB hRmcDll 

DLLxHb FUNCTION RMC_SETTOOLTIPWIDTH( ;       // RMC_SetToolTipWidth
                 nCtrlID, ;
                 nWidth ;
                 ) LIB hRmcDll 

DLLxHb FUNCTION RMC_DELETECHART( nCtrlId ) LIB hRmcDll



*--------------
